@@ -169,7 +169,13 @@ import tokenize
 import operator
 import codecs
 
-from elementtree.ElementTree import ElementTree, XML
+try:
+    # try the python 2.5 way
+    from xml.etree.cElementTree import ElementTree, XML
+except ImportError:
+    # then try the python 2.4 way
+    from elementtree.ElementTree import ElementTree, XML
+
 from turbogears.util import fixentities
 
 __version__ = '1.5'
@@ -179,8 +185,6 @@ DEFAULTKEYWORDS = ', '.join(default_keywords)
 
 EMPTYSTRING = ''
 
-
-
 # The normal pot-file header. msgmerge and Emacs's po-mode work better if it's
 # there.
 pot_header = _('''\
@@ -202,7 +206,6 @@ msgstr ""
 
 ''')
 
-
 def usage(code, msg=''):
     print >> sys.stderr, __doc__ % globals()
     if msg:
@@ -210,7 +213,6 @@ def usage(code, msg=''):
     sys.exit(code)
 
 
-
 escapes = []
 
 def make_escapes(pass_iso8859):
@@ -281,7 +283,6 @@ def normalize(s, escape=False):
         s = re.sub('[\200-\377]', repl, s)
     return s
 
-
 def containsAny(str, set):
     """Check whether 'str' contains ANY of the chars in 'set'"""
     return 1 in [c in str for c in set]
@@ -376,7 +377,6 @@ def getFilesForName(name):
 
     return []
 
-
 class TokenEater:
     def __init__(self, options):
         self.__options = options
@@ -505,17 +505,17 @@ class TokenEater:
 
         if node.text:
             msg = node.text.strip()
-            if msg and not self.contains_inline_python(msg): 
+            if msg and not self.contains_inline_python(msg):
                 if tag not in ['script','style']:
-                    self.__addentry(msg,self.strip_namespace_uri(node.tag), iskidstring=1) 
+                    self.__addentry(msg,self.strip_namespace_uri(node.tag), iskidstring=1)
 
         if node.getchildren():
             for child in node: self.get_text_node(child)
 
         if node.tail:
             msg = node.tail.strip()
-            if msg and not self.contains_inline_python(msg): 
-                self.__addentry(msg,self.strip_namespace_uri(node.tag), iskidstring=1) 
+            if msg and not self.contains_inline_python(msg):
+                self.__addentry(msg,self.strip_namespace_uri(node.tag), iskidstring=1)
 
     def extract_kid_strings(self):
         if not self.__curfile: return
@@ -592,7 +592,6 @@ class TokenEater:
                     print >> fp, 'msgid', normalize(k, options.escape)
                     print >> fp, 'msgstr ""\n'
 
-
 def main():
     global default_keywords
     try:
@@ -753,7 +752,6 @@ def main():
         if closep:
             fp.close()
 
-
 if __name__ == '__main__':
     main()
     # some more test strings
