@@ -3,29 +3,31 @@ import itertools
 import pkg_resources
 import warnings
 from turbogears import view, validators, startup, config
-from turbogears.util import setlike, to_unicode, copy_if_mutable
+from turbogears.util import setlike, to_unicode, copy_if_mutable, get_package_name
+from turbogears.i18n.utils import get_locale
 from turbogears.widgets.meta import MetaWidget, load_kid_template
+from cherrypy.config import configs
 
-try: 
+try:
     set
-except NameError: 
+except NameError:
     from sets import Set as set
 
 __all__ = ["load_widgets", "all_widgets", "Widget", "CompoundWidget",
-           "WidgetsList", "WidgetsDeclaration", "register_static_directory",
-           "static", "Resource", "Link", "CSSLink", "JSLink", 
-           "Source", "CSSSource", "JSSource", "js_location", "mochikit",
+           "WidgetsList", "register_static_directory",
+           "static", "Resource", "Link", "CSSLink", "JSLink",
+           "Source", "CSSSource", "JSSource", "js_location", "mochikit", "jsi18nwidget",
            "WidgetDescription", "set_with_self"]
 
 
 class Enum(set):
-    """Enum used at js_locations which is less strict than 
+    """Enum used at js_locations which is less strict than
     ``turbogears.utils.Enum`` and serves our purposes as well as allowing
-    any object with ``retrieve_javascript``, ``retrieve_css``, and 
+    any object with ``retrieve_javascript``, ``retrieve_css``, and
     ``location`` attributes to provide resources to the template when
     scanned in ``turbogears.controllers._process_output``.
 
-    
+
     Example:
 
         >>> locations = Enum('bodytop', 'bodybottom', head')
@@ -59,13 +61,17 @@ counter = itertools.count()
 # Load all widgets provided by the widget entry point
 def load_widgets():
     for widget_mod in pkg_resources.iter_entry_points("turbogears.widgets"):
-        widget_mod.load()
+        try:
+            widget_mod.load()
+        except Exception, e:
+            raise ImportError, 'Error loading plugin "%s"\n%s: %s' % (
+                widget_mod, e.__class__.__name__, e)
 
 all_widgets = set()
 
 PlainHTML = load_kid_template("""
                               <html xmlns:py="http://purl.org/kid/ns#"
-                                  py:replace="elements" 
+                                  py:replace="elements"
                               />
                               """,
                               modname="turbogears.widgets.plainhtml"
@@ -77,12 +83,12 @@ PlainHTML = load_kid_template("""
 class Widget(object):
     """
     A TurboGears Widget.
-    
+
    '__init__' and 'update_params' are the only methods you might need to
    care extending.
 
     Attributes you should know about:
-    
+
     name      : The name of the widget.
     template  : Kid template for the widget.
     default   : Default value for the widget.
@@ -93,7 +99,7 @@ class Widget(object):
     is_named  : A property returning True if the widget has overrided it's
                 default name.
     params    : All parameter names listed here will be treated as special
-                parameters. This list is updated by the metaclass and 
+                parameters. This list is updated by the metaclass and
                 always contains *all* params from the widget itself and
                 all it's bases. Can be used as a quick reminder of all
                 the params your widget has at it's disposal. They all
@@ -102,26 +108,24 @@ class Widget(object):
     params_doc: A dictionary containing 'params' names as keys and their
                 docstring as value. For documentation at the widget browser.
 
-   All initialization parameters listed at the class attribute "params" can be 
-   defined as class attributes, overriden at __init__ or at display time. They 
+   All initialization parameters listed at the class attribute "params" can be
+   defined as class attributes, overriden at __init__ or at display time. They
    will be treated as special params for the widget, which means:
-   
+
    1) You can fix default values for them when sublcassing Widget.
 
    2) If passed as **params to the constructor, the will be bound automatically
-      to the widget instance, taking preference over any class attributes 
+      to the widget instance, taking preference over any class attributes
       previously defined. Mutable attributes (dicts and lists) defined as class
-      attributes are safe to modify as care is taken to copy them so the class 
-      attribute remains unchanged. 
-      By default, copied dicts are updated with the new values, which means 
-      items from the original class dict will remain there.
+      attributes are safe to modify as care is taken to copy them so the class
+      attribute remains unchanged.
 
-   3) They can be further overrided by passing them as keyword args to 
-      display(). This will only affect that display() call in a thread-safe 
+   3) They can be further overrided by passing them as keyword args to
+      display(). This will only affect that display() call in a thread-safe
       way.
 
    4) A callable can be passed and it will be called automatically when sending
-      variables to the template. This can be handy to pick up parameters which 
+      variables to the template. This can be handy to pick up parameters which
       change in every request or affect many widgets simultaneously.
     """
 
@@ -137,19 +141,19 @@ class Widget(object):
 
     def __init__(self, name=None, template=None, default=None, **params):
         """
-        All initialization has to take place in this method. It's not 
-        thread-safe to mutate widget's attributes outside this method or 
+        All initialization has to take place in this method. It's not
+        thread-safe to mutate widget's attributes outside this method or
         anytime after widget's first display.
 
         *Must* call super(MyWidget, self).__init__(*args, **kw) cooperatively,
         unless, of course, your know what you're doing. Preferably this should
         be done before any actual work is done in the method.
 
-        @params:
+        Parameters:
 
         name     : The widget's name. In input widgets, this will also be the
-                   name of the variable that the form will send to the 
-                   controller. This is the only param that is safe to pass as a 
+                   name of the variable that the form will send to the
+                   controller. This is the only param that is safe to pass as a
                    positional argument to __init__.
         template : The template that the widget should use to display itself.
                    Currently only Kid templates are supported. You can both
@@ -160,6 +164,11 @@ class Widget(object):
         **params : Keyword arguments specific to your widget or to any of it's
                    bases. If listed at class attribute 'params' the will be
                    bound automatically to the widget instance.
+
+        Note: Do not confuse these parameters with parameters listed at
+        "params". Some widgets accept parameters at the constructor which are
+        not listed params, these parameter won't be passed to the template, be
+        automatically called, etc..
         """
         self._declaration_counter = counter.next()
         if name:
@@ -198,9 +207,9 @@ class Widget(object):
 
     def update_params(self, params):
         """
-        This method will have the last chance to update the variables sent to 
-        the template for the specific request. All parameters listed at class 
-        attribute 'params' will be available at the 'params' dict this method 
+        This method will have the last chance to update the variables sent to
+        the template for the specific request. All parameters listed at class
+        attribute 'params' will be available at the 'params' dict this method
         receives.
 
         *Must* call super(MyWidget, self).update_params(params) cooperatively,
@@ -224,7 +233,7 @@ class Widget(object):
 
     def display(self, value=None, **params):
         """
-        Display the widget in a Kid template. Returns an elementtree node 
+        Display the widget in a Kid template. Returns an elementtree node
         instance. If you need serialized output in a string call 'render'
         instead.
         Probably you will not need to override or extend if inhertitting from
@@ -234,9 +243,9 @@ class Widget(object):
 
         value    : The value to display in the widget.
         **params : Extra parameters specific to the widget. All keyword params
-                   supplied will pass through the update_params method which will 
+                   supplied will pass through the update_params method which will
                    have a last chance to modify them before reaching the
-                   template. 
+                   template.
         """
 
         if not getattr(self, 'template_c', False):
@@ -245,27 +254,36 @@ class Widget(object):
         # logic for managing the params attribute
         for param in self.__class__.params:
             if param in params:
-                if callable(params[param]):
-                    params[param] = params[param]()
+                param_value = params[param]
+                if callable(param_value):
+                    param_value = param_value()
             else:
                 # if the param hasn't been overridden (passed as a keyword
                 # argument inside **params) put the corresponding instance
-                # value inside params, also make sure we don't pass a 
-                # reference to mutables
-                params[param] = copy_if_mutable(getattr(self, param, None))
+                # value inside params.
+                param_value = getattr(self, param, None)
+            # make sure we don't pass a reference to mutables
+            params[param] = copy_if_mutable(param_value)
         params["name"] = self.name
         params["value"] = to_unicode(self.adjust_value(value, **params))
         self.update_params(params)
         # update_data has been deprecated
         self.update_data(params)
-        return view.engines.get('kid').transform(params, self.template_c)
+        try:
+            transform = view.engines['kid'].transform
+        except (KeyError, AttributeError):
+            # this can happen if you render a widget before application startup
+            # when view.load_engines() has not yet been called
+            raise RuntimeError("Trying to render a widget,"
+                " but Kid templating engine is not yet loaded.")
+        return transform(params, self.template_c)
 
     def render(self, value=None, format="html", **params):
         """
         Exactly the same as display() but return serialized output instead.
         Useful for debugging or to display the widget in a non-Kid template like
         Cheetah, STAN, ...
-        """ 
+        """
         elem = self.display(value, **params)
         t = PlainHTML(elements=elem)
         return t.serialize(output=format, fragment=True)
@@ -274,7 +292,7 @@ class Widget(object):
         """
         Return a setlike instance with all the JSLinks and JSSources the
         widget needs.
-        """ 
+        """
         scripts = setlike()
         for script in self.javascript:
             scripts.add(script)
@@ -284,7 +302,7 @@ class Widget(object):
         """
         Return a setlike instance with all the CSSLinks and CSSSources the
         widget needs.
-        """ 
+        """
         css = setlike()
         for cssitem in self.css:
             css.add(cssitem)
@@ -313,6 +331,12 @@ class Widget(object):
                 for var in ['name'] + self.__class__.params]))
 
 class CompoundWidget(Widget):
+    """A widget that can contain other widgets.
+
+    A compund widget is a widget that can group several widgets to make a
+    complex widget. Child widget names must be listed at ther widget's
+    ``member_widgets`` attribute.
+    """
     compound = True
     member_widgets = []
 
@@ -326,13 +350,14 @@ class CompoundWidget(Widget):
         super(CompoundWidget, self).__init__(*args, **kw)
 
     def iter_member_widgets(self):
+        """Iteratetes over all the widget's children"""
         for member in self.__class__.member_widgets:
             attr = getattr(self, member, None)
             if isinstance(attr, list):
                 for widget in attr:
                     yield widget
             elif attr is not None:
-                yield attr 
+                yield attr
 
     def display(self, value=None, **params):
         params["member_widgets_params"] = params.copy()
@@ -346,20 +371,10 @@ class CompoundWidget(Widget):
         d['value_for'] = lambda f: self.value_for(f, d['value'])
         widgets_params = d['member_widgets_params']
         d['params_for'] = lambda f: self.params_for(f, **widgets_params)
-        # options_for has been deprecated
-        def deprecated_options_for(item):
-            warnings.warn(
-                "Use of options_for inside your widget's template "
-                "is deprecated, use params_for instead. "
-                "Note: this warning will be removed once 1.0 is "
-                "released and your actual code will stop working.",
-                DeprecationWarning, 2)
-            return self.params_for(item, **widgets_params)
-        d['options_for'] = deprecated_options_for
- 
+
     def value_for(self, item, value):
         """
-        Pick up the value for a given member_widget 'item' from the 
+        Pick up the value for a given member_widget 'item' from the
         value dict passed to this widget.
         """
         name = getattr(item, "name", item)
@@ -367,7 +382,7 @@ class CompoundWidget(Widget):
             return value.get(name)
         else:
             return None
- 
+
     def params_for(self, item, **params):
         """
         Pick up the params for the given member_widget 'item' from
@@ -381,15 +396,6 @@ class CompoundWidget(Widget):
                     item_params[k] = v[name]
         return item_params
 
-    def options_for(self, *args, **kw):
-        warnings.warn(
-            "Use of options_for inside your widget's template "
-            "is deprecated, use params_for instead. "
-            "Note: this warning will be removed once 1.0 is "
-            "released and your actual code will stop working.",
-            DeprecationWarning, 2)
-        return self.params_for(*args, **kw)
-
     def retrieve_javascript(self):
         """
         Retrieve the javascript for all the member widgets and
@@ -453,16 +459,6 @@ class WidgetsList(list):
         if len(self) == 0:
             warnings.warn("You have declared an empty WidgetsList")
 
-class WidgetsDeclaration(WidgetsList):
-    def __init__(self):
-        warnings.warn(
-            "Use of WidgetsDeclaration is deprecated, "
-            "use WidgetsList instead. "
-            "Note: this warning will be removed once 1.0 is "
-            "released and your actual code will stop working.",
-            DeprecationWarning, 2)
-        super(WidgetsDeclaration, self).__init__()
-
 #############################################################################
 # CSS, JS and mochikit stuff                                                #
 #############################################################################
@@ -525,9 +521,9 @@ class Link(Resource):
 
 class CSSLink(Link):
     template = """
-    <link rel="stylesheet" 
-        type="text/css" 
-        href="$link" 
+    <link rel="stylesheet"
+        type="text/css"
+        href="$link"
         media="$media"
     />
     """
@@ -541,8 +537,14 @@ js_location = Enum('head', 'bodytop', 'bodybottom')
 
 class JSLink(Link):
     template = """
-    <script type="text/javascript" src="$link"></script>
+    <script type="text/javascript" src="$link"
+        charset="$charset" defer="${defer and 'defer' or None}"/>
     """
+    params = ["charset", "defer"]
+    params_doc = {'charset': 'The character encoding of the linked script',
+        'defer': 'If true, browser may defer execution of the script'}
+    charset = None
+    defer = False
 
     location = js_location.head
 
@@ -553,13 +555,132 @@ class JSLink(Link):
             if location not in js_location:
                 raise ValueError, "JSLink location should be in %s" % js_location
             self.location = location
-        
-    def add_for_location(self, location):
-        return location == self.location
 
     retrieve_javascript = set_with_self
 
-mochikit = JSLink("turbogears", "js/MochiKit.js")
+class MochiKitLink(JSLink):
+    """Specialized JSLink to allow for suppression of MochiKit inclusion.
+
+    The rationale for this is that MochiKit in version 1.3.1 has some issues
+    that are are remedied by 1.4, but the latter is unfortunately unreleased.
+
+    To remedy this, you can include MochiKit 1.4 by yourself (using e.g.
+    tgMochiKit) - but this will conflict with the contained version which
+    is made use of in several TG widgets.
+
+    By setting the configuration option
+
+      tg.mochikit_suppress = True
+
+    the incusion of MK via built-in widgets will be suppressed. Of course then
+    one has to include the proper version explicitly, or via tg.include_widgets.
+
+    If 'turbogears.mochikit' is part of tg.include_widgets, a warning is issued.
+
+    """
+
+    template = """
+    <script xmlns:py="http://purl.org/kid/ns#" py:if="not suppress" type="text/javascript" src="$link"/>
+    """
+
+    def update_params(self, p):
+        super(MochiKitLink, self).update_params(p)
+        suppress = config.get('tg.mochikit_suppress', False)
+        p['suppress'] = suppress
+        if suppress and 'turbogears.mochikit' in config.get('tg.include_widgets', []):
+            warnings.warn("""\
+tg.mochikit_suppress == True, but 'turbogears.mochikit' is to be included via
+'tg.mochikit_suppress'. This is a contradiction, and mochikit_suppress overrides
+the inclusion to prevent subtle errors due to version mixing of MochiKit.""")
+
+mochikit = MochiKitLink("turbogears", "js/MochiKit.js")
+
+class JSI18NWidget(Widget):
+    """
+    This is a widget that can be used to add support for
+    i18n of javascript texts.
+
+    It will basically add an implementation of the
+    _ function that then can be used to wrap text literals
+    in javascript-code.
+
+    Additionally, message-catalogs are looked up and included
+    depending on the current locale.
+
+    To include the widget, put
+
+    tg.include_widgets = ['turbogears.jsi18nwidget']
+
+    in your application configuration.
+    """
+
+    parameters = ['locale_catalog_providers']
+
+    locale_catalog_providers = []
+
+    def __init__(self, *args, **kwargs):
+        super(JSI18NWidget, self).__init__(*args, **kwargs)
+        self._initialized = False
+
+    def register_package_provider(self, package=None, directory=None, domain=None):
+        parent = self
+        class PackageProvider(object):
+            def __init__(self):
+                self._initialized = False
+
+            def __call__(self, locale):
+                if not self._initialized:
+                    # the leading underscore is to prevent shadowing of the
+                    # register_package_provider-arguments
+                    if package is None:
+                        _package = get_package_name()
+                    else:
+                        _package = package
+                    if directory is None:
+                        _directory = "static/javascript"
+                    else:
+                        _directory = directory
+                    if domain is None:
+                        _domain = config.get('i18n.domain', 'messages')
+                    else:
+                        _domain = domain
+                    js_dir = pkg_resources.resource_filename(_package, _directory)
+                    register_static_directory(_package, js_dir)
+                    self._package_name = _package
+                    self._domain = _domain
+                    self._initialized = True
+                js = []
+                for l in locale, locale[:2]:
+                    link = JSLink(self._package_name, "%s-%s.js" % (self._domain, l))
+                    if parent.linked_file_exists(link):
+                        js.append(link)
+                        break
+                return js
+        self.locale_catalog_providers.append(PackageProvider())
+
+    def retrieve_javascript(self):
+        if not self._initialized:
+            if not config.get('i18n.suppress_default_package_provider', False):
+                self.register_package_provider()
+            self._initialized = True
+        js = super(JSI18NWidget, self).retrieve_javascript()
+        js.add(JSLink("turbogears", 'js/i18n_base.js'))
+        locale = get_locale()
+        for pp in self.locale_catalog_providers:
+            js.extend(pp(locale))
+        return js
+
+    def linked_file_exists(self, widget):
+        key = '/tg_widgets/%s' % widget.mod
+        wp = configs.get(key)
+        if wp:
+            dir = wp.get('static_filter.dir')
+            if dir:
+                path = os.path.join(dir, widget.name)
+                return os.path.exists(path)
+        return False
+
+jsi18nwidget = JSI18NWidget()
 
 class Source(Resource):
     params = ["src"]
@@ -587,19 +708,21 @@ class CSSSource(Source):
 
 class JSSource(Source):
     template = """
-    <script type="text/javascript">$src</script>
+    <script type="text/javascript"
+        defer="${defer and 'defer' or None}">$src</script>
     """
+    params = ["defer"]
+    params_doc = {'defer': 'If true, browser may defer execution of the script'}
+    defer = False
+
     location = js_location.head
 
-    def __init__(self, src, location=None):
+    def __init__(self, src, location=None, **kw):
         if location:
             if location not in js_location:
                 raise ValueError, "JSSource location should be in %s" % js_location
             self.location = location
-        super(JSSource, self).__init__(src)
-
-    def add_for_location(self, location):
-        return location == self.location
+        super(JSSource, self).__init__(src, **kw)
 
     retrieve_javascript = set_with_self
 
@@ -626,7 +749,7 @@ class MetaDescription(MetaWidget):
 
 class WidgetDescription(CompoundWidget):
     """
-    A description for a Widget. Make's the 'for_widget' widget appear in the 
+    A description for a Widget. Make's the 'for_widget' widget appear in the
     browser.
     It's a nice way to show off to your friends your coolest new widgets and
     to have a testing platform while developing them.
@@ -682,10 +805,10 @@ class RenderOnlyWD(WidgetDescription):
         <tt class="rendered">${for_widget.render(value)}</tt>
     </div>
     """
-    
+
     def retrieve_javascript(self):
         return setlike()
-    
+
     def retrieve_css(self):
         return setlike()
 
