@@ -12,25 +12,25 @@ from turbogears.widgets.forms import FormField, CompoundFormField, TextField, \
                                     RadioButtonList
 from turbogears.widgets.rpc import RPC
 
-__all__ = ["CalendarDatePicker", "CalendarDateTimePicker", "AutoCompleteField",
-           "LinkRemoteFunction", "RemoteForm", "AjaxGrid", "URLLink"]
+__all__ = ["CalendarDatePicker", "CalendarDateTimePicker",
+    "AutoCompleteField", "AutoCompleteTextField",
+    "LinkRemoteFunction", "RemoteForm", "AjaxGrid", "URLLink"]
 
 
 class CalendarDatePicker(FormField):
     """Use a Javascript calendar system to allow picking of calendar dates."""
+
     template = """
     <div xmlns:py="http://purl.org/kid/ns#">
     <input type="text" id="${field_id}" class="${field_class}" name="${name}" value="${strdate}" py:attrs="attrs"/>
     <input type="button" id="${field_id}_trigger" class="date_field_button" value="${button_text}"/>
     <script type="text/javascript">
-    Calendar.setup(
-    {
+    Calendar.setup({
         inputField : "${field_id}",
         ifFormat : "${format}",
         button : "${field_id}_trigger"
         <span py:if="picker_shows_time" py:replace="', showsTime : true'"/>
-    }
-    );
+    });
     </script>
     </div>
     """
@@ -39,7 +39,6 @@ class CalendarDatePicker(FormField):
     params_doc = {'attrs': 'Extra attributes',
         'skin': 'For alternate skins, such as "calendar-blue" or "skins/aqua/theme"',
         'picker_shows_time': 'Whether the calendar should let you pick a time, too',
-
         'button_text': 'Text for the button that will show the calendar picker',
         'format': 'The date format (default is mm/dd/yyyy)',
         'calendar_lang': 'The language to be used in the calendar picker.'}
@@ -90,23 +89,58 @@ class CalendarDatePicker(FormField):
 
 
 class CalendarDatePickerDesc(CoreWD):
+
     name = "Calendar"
     for_widget = CalendarDatePicker("date_picker")
 
 
 class CalendarDateTimePicker(CalendarDatePicker):
+    """Javascript calendar system to allow picking of dates with times."""
+
     format = "%Y/%m/%d %H:%M"
     picker_shows_time = True
 
 
 class CalendarDateTimePickerDesc(CoreWD):
+
     name = "Calendar with time"
     for_widget = CalendarDateTimePicker("datetime_picker")
 
 
-class AutoCompleteField(CompoundFormField):
-    """Performs Ajax-style autocompletion by requesting search
-    results from the server as the user types."""
+class AutoComplete(Widget):
+    """Mixin class for autocomplete fields.
+
+    Performs Ajax-style autocompletion by requesting search
+    results from the server as the user types.
+
+    """
+
+    javascript = [mochikit, JSLink(static,"autocompletefield.js")]
+    css = [CSSLink(static,"autocompletefield.css")]
+    params = ["search_controller", "search_param", "result_name", "attrs",
+        "only_suggest", "complete_delay", "take_focus", "min_chars", "show_spinner"]
+    params_doc = {'attrs': 'Extra attributes',
+        'search_controller': 'Name of the controller returning the auto completions',
+        'search_param': 'Name of the search parameter ("*" passes all form fields)',
+        'result_name': 'Name of the result list returned by the controller',
+        'only_suggest': 'If true, pressing enter does not automatically submit the first list item.',
+        'complete_delay': 'Delay (in seconds) before loading new auto completions',
+        'take_focus': 'If true, take focus on load.',
+        'min_chars': 'Minimum number of characters to type before autocomplete activates',
+        'show_spinner': 'If false, the spinner (load indicator) is not shown.'}
+    attrs = {}
+    search_controller = ""
+    search_param = "searchString"
+    result_name = "textItems"
+    only_suggest = False
+    complete_delay = 0.200
+    take_focus = False
+    min_chars = 1
+    show_spinner = True
+
+
+class AutoCompleteField(CompoundFormField, AutoComplete):
+    """Text field with auto complete functionality and hidden key field."""
 
     template = """
     <div xmlns:py="http://purl.org/kid/ns#" id="${field_id}">
@@ -115,10 +149,9 @@ class AutoCompleteField(CompoundFormField):
         '${text_field.field_id}', '${hidden_field.field_id}',
         '${search_controller}', '${search_param}', '${result_name}',${str(only_suggest).lower()},
         '${show_spinner and tg.url([tg.widgets, 'turbogears.widgets/spinner.gif']) or None}',
-        ${complete_delay}, ${str(take_focus).lower()});
+        ${complete_delay}, ${str(take_focus).lower()}, ${min_chars});
         addLoadEvent(AutoCompleteManager${field_id}.initialize);
     </script>
-
     ${text_field.display(value_for(text_field), **params_for(text_field))}
     <img py:if="show_spinner" id="autoCompleteSpinner${field_id}"
         src="${tg.url([tg.widgets, 'turbogears.widgets/spinnerstopped.png'])}" alt=""/>
@@ -126,103 +159,149 @@ class AutoCompleteField(CompoundFormField):
     ${hidden_field.display(value_for(hidden_field), **params_for(hidden_field))}
     </div>
     """
-    javascript = [mochikit, JSLink(static,"autocompletefield.js")]
-    css = [CSSLink(static,"autocompletefield.css")]
+
     member_widgets = ["text_field", "hidden_field"]
-    params = ["search_controller", "search_param", "result_name", "attrs",
-        "only_suggest", "complete_delay", "take_focus", "show_spinner"]
-    params_doc = {'attrs': 'Extra attributes',
-        'search_controller': 'Name of the controller returning the auto completions',
-        'search_param': 'Name of the search parameter ("*" passes all form fields)',
-        'result_name': 'Name of the result list returned by the controller',
-        'only_suggest': 'If true, pressing enter does not automatically submit the first list item.',
-        'complete_delay': 'Delay (in seconds) before loading new auto completions',
-        'take_focus': 'If true, take focus on load.',
-        'show_spinner': 'If false, the spinner (load indicator) is not shown.'}
     text_field = TextField(name="text")
     hidden_field = HiddenField(name="hidden")
-    attrs = {}
-    search_controller = ""
-    search_param = "searchString"
-    result_name = "textItems"
-    only_suggest = False
-    complete_delay = 0.200
-    take_focus = False
-    show_spinner = True
 
 
 class AutoCompleteFieldDesc(CoreWD):
-    name = "Auto Complete"
 
-    states = ["Alabama", "Alaska", "Arizona", "Arkansas", "California",
-              "Colorado", "Connecticut", "Delaware", "Florida",
-              "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana",
-              "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
-              "Maryland", "Massachusetts", "Michigan", "Minnesota",
-              "Mississippi", "Missouri", "Montana", "Nebraska",
-              "Nevada", "New Hampshire", "New Jersey", "New Mexico",
-              "New York", "North Carolina", "North Dakota", "Ohio",
-              "Oklahoma", "Oregon", "Pennsylvania", "Rhode Island",
-              "South Carolina", "South Dakota", "Tennessee", "Texas",
-              "Utah", "Vermont", "Virginia", "Washington",
-              "West Virginia", "Wisconsin", "Wyoming"]
+    name = "AutoCompleteField"
+
+    codes = """AK AL AR AS AZ CA CO CT DC DE FL FM GA GU HI IA ID IL IN KS
+        KY LA MA MD ME MH MI MN MO MP MS MT NC ND NE NH NJ NM NV NY OH
+        OK OR PA PR PW RI SC SD TN TX UM UT VA VI VT WA WI WV WY""".split()
+
+    states = """Alaska Alabama Arkansas American_Samoa Arizona
+        California Colorado Connecticut District_of_Columbia
+        Delaware Florida Federated_States_of_Micronesia Georgia Guam
+        Hawaii Iowa Idaho Illinois Indiana Kansas Kentucky Louisiana
+        Massachusetts Maryland Maine Marshall_Islands Michigan
+        Minnesota Missouri Northern_Mariana_Islands Mississippi
+        Montana North_Carolina North_Dakota Nebraska New_Hampshire
+        New_Jersey New_Mexico Nevada New_York Ohio Oklahoma Oregon
+        Pennsylvania Puerto_Rico Palau Rhode_Island South_Carolina
+        South_Dakota Tennessee Texas U.S._Minor_Outlying_Islands
+        Utah Virginia Virgin_Islands_of_the_U.S. Vermont Washington
+        Wisconsin West_Virginia Wyoming""".split()
+    states = map(lambda s: s.replace('_', ' '), states)
+
+    state_code = dict(zip(codes, states))
 
     template = """
-    <div>
-        Please choose a state:<br/>
-        ${for_widget.display()}
-    </div>
+    <form xmlns:py="http://purl.org/kid/ns#" onsubmit="if (
+        this.elements[0].value &amp;&amp; this.elements[1].value)
+        alert('The alpha code for '+this.elements[0].value
+        +' is '+this.elements[1].value+'.');return false"><table>
+        <tr><th>State</th><td py:content="for_widget.display()"/>
+        <td><input type="submit" value="Show alpha code"/></td></tr>
+    </table></form>
     """
+
     full_class_name = "turbogears.widgets.AutoCompleteField"
 
     def __init__(self, *args, **kw):
         super(AutoCompleteFieldDesc, self).__init__(*args, **kw)
-        self.for_widget = AutoCompleteField(name="state",
-                                    search_controller="%s/search" %
-                                                    self.full_class_name,
-                                    search_param="statename",
-                                    result_name="states")
+        self.for_widget = AutoCompleteField(name="state_and_code",
+            search_controller="%s/search" % self.full_class_name,
+            search_param="state", result_name="states")
 
-    def search(self, statename):
-        statename = statename.lower()
-        return dict(states=
-                filter(lambda item: statename in item.lower(), self.states))
+    def search(self, state):
+        states = []
+        code = state.upper()
+        if code in self.state_code:
+            states.append((self.state_code[code], code))
+        else:
+            states.extend([s for s  in zip(self.states, self.codes)
+                if s[0].lower().startswith(state.lower())])
+        return dict(states=states)
+    search = expose(format="json")(search)
+
+
+class AutoCompleteTextField(TextField, AutoComplete):
+    """Text field with auto complete functionality."""
+
+    template = """
+    <div xmlns:py="http://purl.org/kid/ns#">
+    <script language="JavaScript" type="text/JavaScript">
+        AutoCompleteManager${field_id} = new AutoCompleteManager('${field_id}', '${field_id}', null,
+        '${search_controller}', '${search_param}', '${result_name}', ${str(only_suggest).lower()},
+        '${show_spinner and tg.url([tg.widgets, 'turbogears.widgets/spinner.gif']) or None}',
+        ${complete_delay}, ${str(take_focus).lower()}, ${min_chars});
+        addLoadEvent(AutoCompleteManager${field_id}.initialize);
+    </script>
+    <input type="text" name="${name}" class="${field_class}" id="${field_id}"
+        value="${value}" py:attrs="attrs"/>
+    <img py:if="show_spinner" id="autoCompleteSpinner${field_id}"
+        src="${tg.url([tg.widgets, 'turbogears.widgets/spinnerstopped.png'])}" alt=""/>
+    <div class="autoTextResults" id="autoCompleteResults${field_id}"/>
+    </div>
+    """
+
+
+class AutoCompleteTextFieldDesc(CoreWD):
+
+    name = "AutoCompleteTextField"
+
+    states = AutoCompleteFieldDesc.states
+    state_code = AutoCompleteFieldDesc.state_code
+
+    template = """
+    <table xmlns:py="http://purl.org/kid/ns#">
+        <tr><th>State</th><td py:content="for_widget.display()"/></tr>
+    </table>
+    """
+
+    full_class_name = "turbogears.widgets.AutoCompleteTextField"
+
+    def __init__(self, *args, **kw):
+        super(AutoCompleteTextFieldDesc, self).__init__(*args, **kw)
+        self.for_widget = AutoCompleteTextField(name="state",
+            search_controller="%s/search" % self.full_class_name,
+            search_param="state", result_name="states")
+
+    def search(self, state):
+        states = []
+        code = state.upper()
+        if code in self.state_code:
+            states.append(self.state_code[code])
+        else:
+            states.extend([s for s  in self.states
+                if s.lower().startswith(state.lower())])
+        return dict(states=states)
     search = expose(format="json")(search)
 
 
 class LinkRemoteFunction(RPC):
-    """ Returns a link that executes a POST asynchronously
-    and updates a DOM Object with the result of it """
+    """Link with remote execution.
+
+    Returns a link that executes a POST asynchronously
+    and updates a DOM Object with the result of it.
+
+    """
 
     template = """
-    <a xmlns:py="http://purl.org/kid/ns#" name="${name}" py:attrs="attrs" onclick="${js}" href="#">${value}</a>
+    <a xmlns:py="http://purl.org/kid/ns#" name="${name}"
+        py:attrs="attrs" py:content="value" onclick="${js}" href="#"/>
     """
+
     params = ["attrs"]
     attrs = {}
 
 
 class LinkRemoteFunctionDesc(CoreWD):
-    name = "Ajax remote function"
 
-    states = ["Alabama", "Alaska", "Arizona", "Arkansas", "California",
-              "Colorado", "Connecticut", "Delaware", "Florida",
-              "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana",
-              "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
-              "Maryland", "Massachusetts", "Michigan", "Minnesota",
-              "Mississippi", "Missouri", "Montana", "Nebraska",
-              "Nevada", "New Hampshire", "New Jersey", "New Mexico",
-              "New York", "North Carolina", "North Dakota", "Ohio",
-              "Oklahoma", "Oregon", "Pennsylvania", "Rhode Island",
-              "South Carolina", "South Dakota", "Tennessee", "Texas",
-              "Utah", "Vermont", "Virginia", "Washington",
-              "West Virginia", "Wisconsin", "Wyoming"]
+    name = "AJAX remote function"
+
+    states = AutoCompleteFieldDesc.states
 
     template = """
-    <div>
-        ${for_widget.display("States starting with 'N'", update="items")}
-        <div id="items">states</div>
+    <div id="items">
+        ${for_widget.display("States starting with the letter 'N'", update="items")}
     </div>
     """
+
     full_class_name = "turbogears.widgets.LinkRemoteFunction"
 
     def __init__(self, *args, **kw):
@@ -232,37 +311,35 @@ class LinkRemoteFunctionDesc(CoreWD):
             data = dict(state_starts_with="N"))
 
     def search_linkrf(self, state_starts_with):
-        return '<br/>'.join(
-                filter(lambda item: item.startswith(state_starts_with),
-                       self.states)
-       )
+        return '<br/>'.join(filter(
+            lambda item: item.startswith(state_starts_with), self.states))
     search_linkrf = expose()(search_linkrf)
 
 
 class RemoteForm(RPC, TableForm):
-    """A TableForm that submits the data asynchronously and loads the resulting
-    HTML into a DOM object"""
+    """AJAX table form.
+
+    A TableForm that submits the data asynchronously and loads the resulting
+    HTML into a DOM object
+
+    """
 
     def update_params(self, d):
         super(RemoteForm, self).update_params(d)
         d['form_attrs']['onSubmit'] = "return !remoteFormRequest(this, '%s', %s);" % (
-            d.get("update", ''),
-            jsonify.encode(self.get_options(d)),
-        )
+            d.get("update", ''), jsonify.encode(self.get_options(d)))
 
 
 class RemoteFormDesc(CoreWD):
+
     name = "AJAX Form"
 
     class TestFormFields(WidgetsList):
         name = TextField()
         age = TextField()
         check = CheckBox()
-        radio = RadioButtonList(options=[(1, "Python"),
-                                         (2, "Java"),
-                                         (3, "Pascal"),
-                                         (4, "Ruby")],
-                                default=4)
+        radio = RadioButtonList(options=list(enumerate(
+            "Python Java Pascal Ruby".split())), default=3)
 
     template = """
     <div>
@@ -291,7 +368,7 @@ class RemoteFormDesc(CoreWD):
 ajaxgridcounter = itertools.count()
 
 class AjaxGrid(Widget):
-    """ AJAX updateable datagrid based on widget.js grid. """
+    """AJAX updateable datagrid based on widget.js grid"""
 
     template = """<div id="${id}" xmlns:py="http://purl.org/kid/ns#">
     <a py:if="refresh_text"
@@ -331,7 +408,8 @@ class AjaxGrid(Widget):
 
 
 class AjaxGridDesc(CoreWD):
-    name = "Ajax Grid"
+
+    name = "AJAX Grid"
 
     def facgen(n):
         total = 1
@@ -362,6 +440,8 @@ class AjaxGridDesc(CoreWD):
 
 
 class URLLink(FormField):
+    """Hyperlink"""
+
     template = """
     <a xmlns:py="http://purl.org/kid/ns#"
        href="$link"
@@ -369,6 +449,7 @@ class URLLink(FormField):
        py:attrs="attrs"
     >$text</a>
     """
+
     params = ["target", "text", "link", "attrs"]
     attrs = {}
     params_doc = {'link': 'Hyperlink',
